#include <Application.h>
#include <Bitmap.h>
#include <MediaDefs.h>
#include <MediaFile.h>
#include <MediaTrack.h>

#include <iostream>

int main()
{
	BApplication app("application/media-test");
	int32 width = 640;
	int32 height = 480;
	
	media_format mediaFormat;
	mediaFormat.type = B_MEDIA_RAW_VIDEO;
	mediaFormat.u.raw_video.display.line_width = width;
	mediaFormat.u.raw_video.display.line_count = height;
	mediaFormat.u.raw_video.last_active = mediaFormat.u.raw_video.display.line_count - 1;
	
	size_t pixelChunk;
	size_t rowAlign;
	size_t pixelPerChunk;
	status_t status = get_pixel_size_for(B_RGB32, &pixelChunk,
			&rowAlign, &pixelPerChunk);
	if (status != B_OK)
		return int(status);

	mediaFormat.u.raw_video.display.bytes_per_row = width * rowAlign;			
	mediaFormat.u.raw_video.display.format = B_RGB32;
	mediaFormat.u.raw_video.interlace = 1;	
	mediaFormat.u.raw_video.field_rate = 30;
	mediaFormat.u.raw_video.pixel_width_aspect = 1;
	mediaFormat.u.raw_video.pixel_height_aspect = 1;
	
	media_file_format mediaFileFormat;
	int32 cookie = 0;
	while (get_next_file_format(&cookie, &mediaFileFormat) == B_OK) {
		if (strcmp(mediaFileFormat.pretty_name, "MPEG (Motion Picture Experts Group) format 4") == 0)
			break;
	}
	
	//std::cout << mediaFileFormat.short_name << std::endl;
	media_codec_info codec;
	media_format dummyFormat;
	cookie = 0;
	while (get_next_encoder(&cookie, &mediaFileFormat, &mediaFormat,
			&dummyFormat, &codec) == B_OK) {
		if (strcmp(codec.pretty_name, "MPEG-4 video") == 0)
			break;
	}
	
	entry_ref ref;
	const char* path = "/boot/home/test-crash.mpg";
	status = get_ref_for_path(path, &ref);
	if (status != B_OK)
		return int(status);
	
	for (int i = 0; i < 10000; i++) {
		BMediaFile* file = new (std::nothrow) BMediaFile(&ref, &mediaFileFormat);
		if (file == NULL)
			return int(B_NO_MEMORY);
	
		status = file->InitCheck();
		if (status != B_OK)
			return int(status);
			
		BMediaTrack* mediaTrack = file->CreateTrack(const_cast<media_format*>(&mediaFormat), &codec);
		if (mediaTrack == NULL)
			return int(B_ERROR);
		
		file->CommitHeader();
		
		BRect rect(0, 0, width - 1, height - 1);
		BBitmap* bitmap = new BBitmap(rect, B_RGB32);
		status = mediaTrack->WriteFrames(bitmap->Bits(), 1, B_MEDIA_KEY_FRAME);
		if (status != B_OK)
			return int(status);
		
		delete bitmap;
			
		file->ReleaseAllTracks();
		status = file->CloseFile();
			
		delete file;		// deletes the track, too
	}
	return 0;
}
